__version__ = "1.0"

from .session import make_request
from .types import Mail, MailBox
from .utils import ApiExeption

from typing import List

class CatMail:
    def __init__(self, email: str) -> None:
        """
        Args:
            email (str): the email, example : test@catway.org
        """
        self.email = email
        self.username = email.split("@")[0]

    def get_inboxes(self, limit: int = None, timeout: int = 60) -> List["Mail"]:
        request_url = "https://mail.catway.org/api/{username}/email".format(username=self.username)
        result = make_request(url=request_url, timeout=timeout)
        if not result["ok"]:
            raise ApiExeption(result["error"])

        _ = []
        for i in result["data"]["mails"]:
            if limit is not None and len(_) == limit:
                break
            _.append(Mail.from_json(i))

        return _

    def get_inbox(self, id: str, timeout: int = None) -> "MailBox":
        request_url = "https://mail.catway.org/api/{id}/inbox".format(id=id)
        result = make_request(url=request_url, timeout=timeout)
        if not result["ok"]:
            raise ApiExeption(result["error"])
        return MailBox.from_json(result["data"])

    def __enter__(self):
        return self

    def __exit__(self, *args):
        return self


from pydantic import BaseModel
from datetime import datetime
from typing_extensions import TypedDict

from .utils import convert_to_datetime, convert_to_str

class Object(BaseModel):
    def __str__(self: "Object"):
        return self.model_dump_json(indent=4)

class MailDict(TypedDict):
    id: "str"
    senderEmail: "str"
    senderName: "str"
    subject: "str"
    createdAt: "str"
    updatedAt: "str"
    expireAt: "str"

class Mail(Object):
    id: str
    sender_email: "str"
    sender_name: "str"
    subject: "str"
    created_at: "datetime"
    updated_at: "datetime"
    expire_at: "datetime"

    @staticmethod
    def from_json(_d: "MailDict") -> "Mail":
        to_parse = {
            "id": _d.get("id"),
            "sender_email": _d.get("senderEmail"),
            "sender_name": _d.get("senderName"),
            "subject": _d.get("subject"),
            "created_at": convert_to_datetime(_d.get("createdAt")),
            "updated_at": convert_to_datetime(_d.get("updatedAt")),
            "expire_at": convert_to_datetime(_d.get("expireAt"))
        }
        return Mail(**to_parse)

    @property
    def view_link(self) -> "str":
        return "https://mail.catway.org/inbox/{id}".format(id=self.id)

    @property
    def raw(self) -> "MailDict":
        return {
            "id": self.id,
            "senderEmail": self.sender_email,
            "senderName": self.sender_name,
            "subject": self.subject,
            "createdAt": convert_to_str(self.created_at),
            "updatedAt": convert_to_str(self.updated_at),
            "expireAt": convert_to_str(self.expire_at)
        }

class MailBoxDict(TypedDict):
    id: str
    html: str
    content: str
    senderEmail: "str"
    senderName: "str"
    subject: "str"
    createdAt: "str"
    updatedAt: "str"
    expireAt: "str"
    mailboxOwner: "str"

class MailBox(Object):
    id: "str"
    html: "str"
    content: "str"
    sender_email: "str"
    sender_name: "str"
    subject: "str"
    created_at: "datetime"
    updated_at: "datetime"
    expire_at: "datetime"
    owner: "str"

    @staticmethod
    def from_json(_d: "MailBoxDict") -> "MailBox":
        to_parse = {
            "id": _d.get("id"),
            "html": _d.get("html"),
            "content": _d.get("content"),
            "sender_email": _d.get("senderEmail"),
            "sender_name": _d.get("senderName"),
            "subject": _d.get("subject"),
            "created_at": convert_to_datetime(_d.get("createdAt")),
            "updated_at": convert_to_datetime(_d.get("updatedAt")),
            "expire_at": convert_to_datetime(_d.get("expireAt")),
            "owner": _d.get("mailboxOwner"),
        }
        return MailBox(**to_parse)

    @property
    def view_link(self) -> "str":
        return "https://mail.catway.org/inbox/{id}".format(id=self.id)

    @property
    def raw(self) -> "MailBoxDict":
        return {
            "id": self.id,
            "html": self.html,
            "content": self.content,
            "senderEmail": self.sender_email,
            "senderName": self.sender_name,
            "subject": self.subject,
            "createdAt": convert_to_str(self.created_at),
            "updatedAt": convert_to_str(self.updated_at),
            "expireAt": convert_to_str(self.expire_at),
            "mailboxOwner": self.owner
        }


from datetime import datetime
from typing import Union

__all__ = [
    "str_to_datetime",
    "convert_to_datetime",
    "convert_to_str",
    "ApiExeption"
]

class ApiExeption(Exception):
    pass

def str_to_datetime(_s: str) -> "datetime":
    return datetime.strptime(_s, "%Y-%m-%dT%H:%M:%S.%fZ")

def convert_to_datetime(_s: str = None) -> Union["datetime", "None"]:
    if _s is not None:
        return str_to_datetime(_s)
    else:
        return None

def convert_to_str(_d: datetime = None) -> "str":
    if _d is not None:
        return str(_d)
    else:
        return None


from .async_session import process_request, session_manager
from .types import Mail, MailBox
from .utils import ApiExeption

from typing import AsyncGenerator, Any

class CatMail:
    def __init__(self, email: str) -> None:
        """
        Args:
            email (str): the email, example : test@catway.org
        """
        self.email = email
        self.username = email.split("@")[0]

    async def get_inboxes(self, limit: int = None, timeout: int = 60) -> AsyncGenerator["Mail", Any]:
        request_url = "https://mail.catway.org/api/{username}/email".format(username=self.username)
        result = await process_request(url=request_url, timeout=timeout)
        if not result["ok"]:
            raise ApiExeption(result["error"])

        count = 0

        for i in result["data"]["mails"]:
            if limit is not None and count == limit:
                return
            count += 1
            yield Mail.from_json(i)

    async def get_inbox(self, id: str, timeout: int = None) -> "MailBox":
        request_url = "https://mail.catway.org/api/{id}/inbox".format(id=id)
        result = await process_request(url=request_url, timeout=timeout)
        if not result["ok"]:
            raise ApiExeption(result["error"])
        return MailBox.from_json(result["data"])

    async def __aenter__(self):
        return self

    async def __aexit__(self, *args):
        session = await session_manager.get_session()
        return await session.close()


import requests
import threading
from datetime import datetime

thread_local = threading.local()
SESSION_TIME_TO_LIVE = 600

session = None


def per_thread(key, construct_value, reset=False):
    """
    :meta private:
    """
    if reset or not hasattr(thread_local, key):
        value = construct_value()
        setattr(thread_local, key, value)

    return getattr(thread_local, key)

def _get_req_session(reset=False):
    if SESSION_TIME_TO_LIVE:
        # If session TTL is set - check time passed
        creation_date = per_thread('req_session_time', lambda: datetime.now(), reset)
        # noinspection PyTypeChecker
        if (datetime.now() - creation_date).total_seconds() > SESSION_TIME_TO_LIVE:
            # Force session reset
            reset = True
            # Save reset time
            per_thread('req_session_time', lambda: datetime.now(), True)

    if SESSION_TIME_TO_LIVE == 0:
        # Session is one-time use
        return requests.sessions.Session()
    else:
        # Session lives some time or forever once created. Default
        return per_thread('req_session', lambda: session if session else requests.sessions.Session(), reset)

def make_request(url: str, timeout: int = 60) -> dict:
    result = _get_req_session().request(
        method="get",
        url=url,
        timeout=timeout
    )
    return result.json()


import aiohttp
import ssl
import certifi

class SessionManager:
    def __init__(self) -> None:
        self.session = None
        self.ssl_context = ssl.create_default_context(cafile=certifi.where())


    async def create_session(self):
        self.session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(
            limit=50,
            ssl_context=self.ssl_context
        ))
        return self.session

    async def get_session(self):
        if self.session is None:
            self.session = await self.create_session()
            return self.session

        if self.session.closed:
            self.session = await self.create_session()

        # noinspection PyProtectedMember
        if not self.session._loop.is_running():
            await self.session.close()
            self.session = await self.create_session()
        return self.session


session_manager = SessionManager()

async def process_request(url: str, timeout: int = 60,) -> dict:
    session = await session_manager.get_session()
    async with session.request(
        method="get",
        url=url,
        timeout=aiohttp.ClientTimeout(total=timeout),
    ) as response:
        return await response.json()
